/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


 *   The WagnisGun, WagnisProjectile and WagnisCursor was made to be used as a "cursor", by fireing invisible projectiles at an object.
 *   You can then monitor the Objects HP in order to know which one has been hit (=clicked). And yes, it isn't a very beautiful solution.
*/

#include "WagnisGun.h"

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "core/command/Executor.h"

#include "graphics/Model.h"
#include "weaponsystem/Weapon.h"
#include "weaponsystem/WeaponPack.h"
#include "weaponsystem/WeaponSystem.h"
#include "worldentities/WorldEntity.h"
#include "worldentities/pawns/Pawn.h"

#include "weapons/projectiles/WagnisProjectile.h"
#include "weapons/MuzzleFlash.h"

namespace orxonox
{
    RegisterClass(WagnisGun);

    WagnisGun::WagnisGun(Context* context) : WeaponMode(context)
    {
        RegisterObject(WagnisGun);

        this->reloadTime_ = 0.5f;
        this->damage_ = 0.0f; //default 15
        this->speed_ = 800.0f;
        this->delay_ = 0.0f;
        this->setMunitionName("LaserMunition");



        this->delayTimer_.setTimer(this->delay_, false, createExecutor(createFunctor(&WagnisGun::shot, this)));
        this->delayTimer_.stopTimer();
        hudImageString_ = "Orxonox/WSHUD_WM_HsW01";
    }

    WagnisGun::~WagnisGun()
    {
    }

    void WagnisGun::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(WagnisGun, XMLPort, xmlelement, mode);

        XMLPortParam(WagnisGun, "delay", setDelay, getDelay, xmlelement, mode);
        XMLPortParam(WagnisGun, "material", setMaterial, getMaterial, xmlelement, mode);
        XMLPortParam(WagnisGun, "projectileMesh", setMesh, getMesh, xmlelement, mode);
        XMLPortParam(WagnisGun, "sound", setSound, getSound, xmlelement, mode);
    }

    /**
    @brief
        Set the firing delay.
    @param delay
        The firing delay in seconds.
    */
    void WagnisGun::setDelay(float delay)
    {
        this->delay_ = delay;
        this->delayTimer_.setInterval(this->delay_);
    }

    void WagnisGun::fire()
    {
        this->delayTimer_.startTimer();
    }

    /**
    @brief
        Fires the weapon. Creates a projectile and fires it.
    */
    void WagnisGun::shot()
    {
        assert( this->getWeapon() && this->getWeapon()->getWeaponPack() && this->getWeapon()->getWeaponPack()->getWeaponSystem() && this->getWeapon()->getWeaponPack()->getWeaponSystem()->getPawn() );

        // Create the projectile.
        WagnisProjectile* projectile = new WagnisProjectile(this->getContext());

        this->computeMuzzleParameters(this->getWeapon()->getWeaponPack()->getWeaponSystem()->getPawn()->getAimPosition());
        projectile->setOrientation(this->getMuzzleOrientation());
        projectile->setPosition(this->getMuzzlePosition());
        projectile->setVelocity(this->getMuzzleDirection() * this->speed_);

        projectile->setShooter(this->getWeapon()->getWeaponPack()->getWeaponSystem()->getPawn());
        projectile->setDamage(this->getDamage());
        projectile->setShieldDamage(this->getShieldDamage());
        projectile->setHealthDamage(this->getHealthDamage());
    }
}
